;;*****************************************************************************
;;*****************************************************************************
;;  FILENAME:   LCD_1.asm
;;  Version: 1.2, Updated on 2003/11/18 at 15:01:19
;;  Generated by PSoC Designer ver 4.1 b931 : 21 January, 2004
;;
;;  DESCRIPTION: LCD User Module software implementation file
;;               for 22/24/25/26/27xxx PSoC family of devices.
;;
;; This set of functions is written for the common 2 and 4 line
;; LCDs that use the Hitachi HD44780A controller.
;;
;;  LCD connections to PSoC port
;;
;;    PX.0 ==> LCD D4
;;    PX.1 ==> LCD D5
;;    PX.2 ==> LCD D6
;;    PX.3 ==> LCD D7
;;    PX.4 ==> LCD E
;;    PX.5 ==> LCD RS
;;    PX.6 ==> LCD R/W
;;
;;  NOTE: User Module APIs conform to the fastcall convention for marshalling
;;        arguments and observe the associated "Registers are volatile" policy.
;;        This means it is the caller's responsibility to preserve any values
;;        in the X and A registers that are still needed after the API
;;        function returns. Even though these registers may be preserved now,
;;        there is no guarantee they will be preserved in future releases.
;;-----------------------------------------------------------------------------
;;  Copyright (c) Cypress MicroSystems 2001-2003. All Rights Reserved.
;;*****************************************************************************
;;*****************************************************************************

include "m8c.inc"
include "LCD_1.inc"

;-----------------------------------------------
;  Global Symbols
;-----------------------------------------------

export   LCD_1_Start
export  _LCD_1_Start
export   LCD_1_Init
export  _LCD_1_Init

export   LCD_1_WriteData
export  _LCD_1_WriteData

export   LCD_1_Control
export  _LCD_1_Control

export  LCD_1_PrString
export _LCD_1_PrString

export  LCD_1_PrCString
export _LCD_1_PrCString

export  LCD_1_Position
export _LCD_1_Position

export  LCD_1_PrHexByte
export _LCD_1_PrHexByte

export  LCD_1_PrHexInt
export _LCD_1_PrHexInt

export  LCD_1_Delay50uTimes
export _LCD_1_Delay50uTimes

export  LCD_1_Delay50u
export _LCD_1_Delay50u


;************************************
; Do not use.  Will be removed in
; future version.
export   LCD_1_Write_Data
export  _LCD_1_Write_Data
;************************************
;-----------------------------------------------
; If bargraph functions not required, don't
; export the function names.
;-----------------------------------------------

IF (LCD_1_BARGRAPH_ENABLE)
export  LCD_1_InitBG
export _LCD_1_InitBG

export  LCD_1_InitVBG
export _LCD_1_InitVBG

export  LCD_1_DrawVBG
export _LCD_1_DrawVBG

export  LCD_1_DrawBG
export _LCD_1_DrawBG
ENDIF

;-----------------------------------------------
;  EQUATES
;-----------------------------------------------
LCD_E:              equ    10h
LCD_RW:             equ    40h
LCD_RS:             equ    20h

LCD_DATA_MASK:      equ    0Fh
LCD_READY_BIT:      equ    08h

LCD_DATA_READ:      equ    ( LCD_E | LCD_RW | LCD_RS )
LCD_CNTL_READ:      equ    ( LCD_E | LCD_RW )
LCD_PORT_WRITE:     equ    7Fh
LCD_PORT_MASK:      equ    7Fh

LCD_Port:           equ    PRT1DR
LCD_PortMode0:      equ    PRT1DM0
LCD_PortMode1:      equ    PRT1DM1

DISP_INC:           equ    03h
DISP_OFF:           equ    08h
DISP_ON:            equ    0Ch
LCD_4BIT_2LINE:     equ    2Ch


;-----------------------------------------------
;      Bargraph definitions
;-----------------------------------------------

LCD_BG_CHAR_WIDTH:  equ    16     ; 16 characters in width
LCD_BG_SEG_WIDTH:   equ    80     ; 16 * 5 = 80
LCD_BG_COL_START:   equ     0     ; Always start in the left most column

                                  ; Offsets for 2x16, 2x20, 4x20
                                  ; Change these values for a custome LCD

LCD_ROW1_OFFSET:    equ    80h    ; Address/command offset for row 1
LCD_ROW2_OFFSET:    equ    C0h    ; Address/command offset for row 2
LCD_ROW3_OFFSET:    equ    94h    ; Address/command offset for row 1
LCD_ROW4_OFFSET:    equ    D4h    ; Address/command offset for row 2

LCD_BG_ROW1_OFFSET: equ    80h    ; Address/command offset for row 1
LCD_BG_ROW2_OFFSET: equ    C0h    ; Address/command offset for row 2

CG_RAM_OFFSET:      equ    40h    ; Offset to character RAM

AREA UserModules (ROM, REL)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_PrCString
;
;  DESCRIPTION:
;    Print constant (ROM) string to LCD
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A:X  Pointer to String
;          A contains MSB of string address
;          X contains LSB of string address
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
 LCD_1_PrCString:
_LCD_1_PrCString:

 Loop_PrCString:
    push  A                            ; Store ROM pointer
    push  X
    romx                               ; Get character from ROM
    jnz   LCD_PrCString_WR             ; print character and advance pointer
    pop   X                            ; Restore the stack
    pop   A
    ret                                ; Return

LCD_PrCString_WR:
    call  LCD_1_WriteData              ; Write data to LCD
    call  LCD_Check_Ready              ; Wait for LCD to be ready
    pop   X                            ; Get ROM pointer
    pop   A
    inc   X                            ; Inc LSB of pointer
    jnc   Loop_PrCString
    inc   A                            ; Inc MSB of pointer if LSB overflow
    jmp   Loop_PrCString

 ;End_LCD_PrCString:
 ;      ret
.ENDSECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_PrHexByte
;
;  DESCRIPTION:
;     Print a byte in Hex (two characters) to current LCD position
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A  => (BYTE) Data/char to be printed
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
.LITERAL
LCD_HEX_STR::
     DS    "0123456789ABCDEF"
.ENDLITERAL
.SECTION

 LCD_1_PrHexByte:
_LCD_1_PrHexByte:

    push  A                            ; Save lower nibble
    asr   A                            ; Shift high nibble to right
    asr   A
    asr   A
    asr   A
    and   A,0Fh                        ; Mask off nibble
    index LCD_HEX_STR                  ; Get Hex value
    call  LCD_1_WriteData              ; Write data to screen
    pop   A                            ; Restore value
    and   A,0Fh                        ; Mask off lower nibble
    index LCD_HEX_STR                  ; Get Hex value
    call  LCD_1_WriteData              ; Write data to screen
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_PrHexInt
;
;  DESCRIPTION:
;     Print an Int in Hex (four characters) to current LCD position
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A:X Integer value
;         A  contains LSB of Int
;         X  contains MSB of Int
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------

 LCD_1_PrHexInt:
_LCD_1_PrHexInt:
    swap  A,X
    call  LCD_1_PrHexByte              ; Print MSB
    mov   A,X                          ; Move LSB into position
    call  LCD_1_PrHexByte              ; Print LSB
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_PrString
;
;  DESCRIPTION:
;     Print (RAM) ASCII string to LCD
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A:X contains pointer to string
;         X  contains LSB of string pointer
;         A  contains MSB or page of string pointer (not used at this time)
;
;  RETURNS:
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;        Currently the MSB of the pointer to the string
;        is assumed to be 0 (bank 0)
;
;-----------------------------------------------------------------------------
 LCD_1_PrString:
_LCD_1_PrString:

 Loop_PrString:
    mov   A,[X]                        ; Get value pointed to by X
    jz    End_LCD_PrString             ; Check for end of string
    call  LCD_1_WriteData              ; Write data to screen
    inc   X                            ; Advance pointer to next character
    push  X
    call  LCD_Check_Ready
    pop   X
    jmp   Loop_PrString                ; Go get next character
End_LCD_PrString:
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_WriteData
;
;  DESCRIPTION:
;     Write a byte to the LCD's data register.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A contains byte to be written to LCD data register
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
 LCD_1_WriteData:
_LCD_1_WriteData:
 LCD_1_Write_Data:   ; Do not use
_LCD_1_Write_Data:   ; Do not use

    push  A                            ; Save copy of character
    asr   A                            ; Shift high nibble to right
    asr   A
    asr   A
    asr   A
    and   A,0Fh                        ; Mask off high nibble
    call  LCD_WDATA_Nibble             ; Write Upper nibble
    pop   A                            ; Retrieve copy of character
    and   A,0Fh                        ; Mask off high nibble
    nop
    nop
    nop
    call  LCD_WDATA_Nibble             ; Write Lower nibble
    call  LCD_Check_Ready
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_Control
;
;  DESCRIPTION:
;     Write a byte to the LCD's control register.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A contains data to be written to LCD control register.
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
 LCD_1_Control:
_LCD_1_Control:

    push  A                            ; Save copy of byte
    asr   A                            ; Shift Upper Nibble to right
    asr   A
    asr   A
    asr   A
    and   A,0Fh                        ; Mask off, just in case
    call  LCD_WCNTL_Nibble             ; Write high nibble
    pop   A                            ; Restore copy of byte
    and   A,0Fh                        ; Mask off high nibble
    nop
    nop
    nop
    call  LCD_WCNTL_Nibble             ; Write Lower nibble
    call  LCD_Check_Ready
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_WCNTL_Nibble
;
;  DESCRIPTION:
;     Write a single nibble to the LCD's command register
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A[3:0]   Contains Nibble to be written to command register
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
 LCD_WCNTL_Nibble:
;   mov   reg[LCD_Port], 00h                     ; Reset control lines
    push  A
    and   [Port_1_Data_SHADE],~LCD_PORT_MASK
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A                        ; Reset control lines

    pop   A
    and   A,LCD_DATA_MASK                        ; Make sure no bogus data in MSN
    or    A,LCD_E                                ; Bring "E" Enable line high
    or    A,[Port_1_Data_SHADE]                  ; OR in bit 7 just
    mov   reg[LCD_Port], A                       ; Write data
    mov   [Port_1_Data_SHADE],A                  ; Keep shadow register in sync
    nop
    and   A,(~LCD_PORT_MASK|LCD_DATA_MASK)       ; Disable E signal and leave data on bus.
    mov   [Port_1_Data_SHADE],A                  ; Keep shadow register in sync
    mov   reg[LCD_Port],A
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_WDATA_Nibble
;
;  DESCRIPTION:
;     Write a single nibble to the LCD's DATA register
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A[3:0]   Contains Nibble to be written to data register
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
 LCD_WDATA_Nibble:

    push  A
    and   [Port_1_Data_SHADE],~LCD_PORT_MASK
    or    [Port_1_Data_SHADE],LCD_RS                  ; Raise RS to signify a Data Write
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A

    pop   A
    and   A,LCD_DATA_MASK                             ; Make sure no bogus data in A[7:4]
    or    A,(LCD_E | LCD_RS)                          ; Bring "E" Enable line high
    or    A,[Port_1_Data_SHADE]                       ; Keep shadow in sync
    mov   reg[LCD_Port], A                            ; Write data
    mov   [Port_1_Data_SHADE],A                       ; Keep shadow in sync
    NOP
    and   A,(~LCD_PORT_MASK|LCD_DATA_MASK|LCD_RS)     ; Disable E signal and leave Data on bus
    mov   [Port_1_Data_SHADE],A                       ; keep shadow in sync
    mov   reg[LCD_Port],A
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_Check_Ready
;
;  DESCRIPTION:
;     Wait until LCD has completed last command.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;     If LCD is not present, this routine may never return.
;
;-----------------------------------------------------------------------------
 LCD_Check_Ready:
    push  A                                                     ; Save Accumulator
    and   [Port_1_Data_SHADE],~LCD_PORT_MASK                    ; Mask of all LCD bits
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A                                       ; Zero LCD port bits

    and   [Port_1_DriveMode_0_SHADE],~LCD_PORT_MASK             ; Clear out LCD mode bits.
    or    [Port_1_DriveMode_0_SHADE],~LCD_READY_BIT             ; Enable data bits to read.
    mov   A,[Port_1_DriveMode_0_SHADE]
    M8C_SetBank1                                                ; Change port mode to read status
    mov   reg[LCD_PortMode0],A                                  ; Setup LCD Port for reading
    M8C_SetBank0

    or    [Port_1_Data_SHADE],LCD_RW                            ; Raise RW to signify Read operation
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A
    NOP

LCD_RDY_LOOP:
    or    [Port_1_Data_SHADE], LCD_CNTL_READ                    ; Raise E to start cycle
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A

    nop                                                         ; Wait 2 nops to make sure data is ready
    nop
    mov   A,reg[LCD_Port]

IF(ICE_PORT_SYNC)                          ; This code is to work around a bug in the ICE
    mov   reg[ 0xfa], A                    ; when an async signal is read through an IO port.
    mov   A, reg[0xfa]                     ; This code is not required when running without
ENDIF                                      ; the ICE.
    push  A
    and   [Port_1_Data_SHADE],(~LCD_PORT_MASK | LCD_RW)         ; Lower E signal
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A

    nop                                    ; Add delay for the slowest part and the
    nop                                    ; fastest PSoC
    nop
                                                                ; Get the LSBs
    or    [Port_1_Data_SHADE],LCD_CNTL_READ                     ; Raise E to start cycle
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A

    nop
    nop

    and   [Port_1_Data_SHADE],(~LCD_PORT_MASK | LCD_RW)         ; Lower E signal
    mov   A,[Port_1_Data_SHADE]
    mov   reg[LCD_Port],A

    pop   A
    and   A,LCD_READY_BIT                                       ; Check busy
    jnz   LCD_RDY_LOOP                                          ; If LCD still busy, read again

    and   [Port_1_DriveMode_0_SHADE],~LCD_PORT_MASK             ; Mask off LCD bits
    or    [Port_1_DriveMode_0_SHADE],LCD_PORT_WRITE             ; Lower R/W signal
    mov   A,[Port_1_DriveMode_0_SHADE]
    M8C_SetBank1
    mov   reg[LCD_PortMode0],A                                  ; Setup LCD Port for writing
    M8C_SetBank0
    pop   A                                                     ; Restore Accumulator
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_Start
;  FUNCTION NAME: LCD_1_Init
;
;  DESCRIPTION:
;     Initialize LCD
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;
;  THEORY of OPERATION or PROCEDURE:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;    This initialization is a bit long, but it should work for
;    most 2 and 4 line LCDs.
;
;-----------------------------------------------------------------------------
 LCD_1_Start:
_LCD_1_Start:
 LCD_1_Init:
_LCD_1_Init:

    and   [Port_1_DriveMode_0_SHADE],~LCD_PORT_MASK             ; Mask off LCD bits
    or    [Port_1_DriveMode_0_SHADE],LCD_PORT_WRITE             ; Set LCD port for writing
    and   [Port_1_DriveMode_1_SHADE],~LCD_PORT_MASK             ; Mask off LCD bits

    mov   A,[Port_1_DriveMode_0_SHADE]
    M8C_SetBank1
    mov   reg[LCD_PortMode0],A                                  ; Setup LCD Port for writing
    mov   A,[Port_1_DriveMode_1_SHADE]
    mov   reg[LCD_PortMode1],A
    M8C_SetBank0

    mov   A,250                             ; Delay for 12.5 mSec (250 * 50uSec)
    call  LCD_1_Delay50uTimes
    mov   A,250                             ; Delay for 12.5 mSec (250 * 50uSec)
    call  LCD_1_Delay50uTimes

    mov   A,03h
    call  LCD_WCNTL_Nibble

    mov   A,82                              ; Delay for 4.1 mSec (82 * 50uSec)
    call  LCD_1_Delay50uTimes

    mov   A,03h
    call  LCD_WCNTL_Nibble

    call  LCD_1_Delay50u
    call  LCD_1_Delay50u
    call  LCD_1_Delay50u

    mov   A,03h
    call  LCD_WCNTL_Nibble

    mov   A,90                              ; Delay for 4.5 mSec (90 * 50uSec)
    call  LCD_1_Delay50uTimes

    mov   A,02h
    call  LCD_WCNTL_Nibble

    mov   A,90                              ; Delay for 4.5 mSec (90 * 50uSec)
    call  LCD_1_Delay50uTimes

    mov   A,08h
    call  LCD_1_Control
    mov   A,90                              ; Delay for 4.5 mSec (90 * 50uSec)
    call  LCD_1_Delay50uTimes

    mov   A,01h
    call  LCD_1_Control
    mov   A,90                              ; Delay for 4.5 mSec (90 * 50uSec)
    call  LCD_1_Delay50uTimes

    mov   A,06h
    call  LCD_1_Control

    mov   A,0Eh
    call  LCD_1_Control

    mov   A,LCD_4BIT_2LINE                  ; Setup for 4 bit interface, 2 line
    call  LCD_1_Control

    mov   A,DISP_OFF
    call  LCD_1_Control

    mov   A,DISP_ON
    call  LCD_1_Control

    mov   A,DISP_INC
    call  LCD_1_Control

    mov   A,90                              ; Delay for 4.5 mSec (90 * 50uSec)
    call  LCD_1_Delay50uTimes

    ret
.ENDSECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_Position
;
;  DESCRIPTION:
;     Position Cursor at Row and Col location
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A => Row  0 to 3
;     X => Col  0 to 39+
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;-----------------------------------------------------------------------------
.LITERAL
LCD_ROW_OFFSET::
     DB    LCD_ROW1_OFFSET, LCD_ROW2_OFFSET, LCD_ROW3_OFFSET, LCD_ROW4_OFFSET
.ENDLITERAL

.SECTION
 LCD_1_Position:
_LCD_1_Position:

    and   A,03h                        ; Mask off 2 bits for row address 0 to 3
    push  X                            ; Store COL
    index LCD_ROW_OFFSET               ; Get ROW memory offset from table
    mov   X,SP                         ; Get Stack pointer
    add   A,[X+(-1)]                   ; Add the COL to the display pointer
    pop   X

LCD_POS_IT:
    call  LCD_1_Control                ; Write control byte
    call  LCD_Check_Ready              ; Wait for LCD to be ready
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_Delay50uTimes
;
;  DESCRIPTION:
;     Delay increments of 50uSeconds
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A contains the delay multiplier
;
;  RETURNS:
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;
;-----------------------------------------------------------------------------
 LCD_1_Delay50uTimes:
_LCD_1_Delay50uTimes:
    call  LCD_1_Delay50u
    dec   A
    jnz   LCD_1_Delay50uTimes
    ret

.ENDSECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_Delay50u
;
;  DESCRIPTION:
;     Delay 50uSec for any clock frequency from 1.5MHz to 24MHz
;     Slower clock frequencies the delay will be;
;           1.5
;        -------------- * 50uSec
;        clock_freq(MHz)
;
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS: none
;
;  RETURNS: none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;
;  THEORY of OPERATION or PROCEDURE:
;
;-----------------------------------------------------------------------------
.LITERAL
 Delay50u_Table::
     DB    08h,  19h,   3Ah,   7Ch,   01h,    01h,    01h,   01h
;         3MHz, 6MHz, 12MHz, 24MHz, 1.5MHz, 750kHz, 188kHz, 94kHz
.ENDLITERAL
.SECTION

  LCD_1_Delay50u:
 _LCD_1_Delay50u:                      ; [11]  Call
    push  A
    M8C_SetBank1                       ; [4]
    mov   A, reg[OSC_CR0]              ; [6] Get delay value
    M8C_SetBank0                       ; [4]
    and   A,07h                        ; [4] Mask off only the clock bits
    cmp   A,05h
    jnc   Delay50u_End
    index Delay50u_Table               ; [13] Get delay value
Delay50u_Loop:                         ;
    dec   A                            ; [4]
    jnz   Delay50u_Loop                ; [5]
Delay50u_End:
    pop   A
    ret
.ENDSECTION


;-----------------------------------------------------------------------------
;      If bargraph is not enabled, the following functions are not required.
;-----------------------------------------------------------------------------



IF (LCD_1_BARGRAPH_ENABLE)

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_DrawBG
;
;  DESCRIPTION:
;     Draw a horizontal bargraph on the LCD with the given parameters.
;
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;    A    => Starting row for bargraph 0 to 3
;   [X]   => Starting Column for bargraph 0 to 39+
;   [x-1] => Length of bargraph in chars 1 to 40+
;   [X-2] => Position of pointer in segments 5 times Length
;
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;    If LCD_1_Init is not called before this function, the
;    bargraph will not be drawn properly.
;
;-----------------------------------------------------------------------------

; Stack offset constants
BG_COL:       equ  0                   ; Stack position of Column
BG_CHAR_LEN:  equ -1                   ; Stack position of Length
BG_LENGTH:    equ -2                   ; Stack postion of bargraph pointer position


  LCD_1_DrawBG:
 _LCD_1_DrawBG:
    push  X
    mov   X,[X+BG_COL]                 ; Row in A, Col in X
    call  LCD_1_Position               ; Set cursor position
    pop   X                            ; Restore pointer

 LCD_BG_LOOP1:
    cmp   [X+BG_LENGTH],00h            ; Check for past end of BG
    jnz   LCD_CHECK1
    mov   A,00h                        ; Load empty character
    jmp   LCD_BG_DOIT                  ;

LCD_CHECK1:
    cmp   [X+BG_LENGTH],06h            ; Check if BG pointer is at this character
    jnc   LCD_CHECK2                   ; Note yet, use full character
    mov   A,[X+BG_LENGTH]
    sub   [X+BG_LENGTH],A
    jmp   LCD_BG_DOIT

LCD_CHECK2:                            ; Put index to full character
    mov   A, 06h
    sub   [X+BG_LENGTH],05h            ; Subtract another 5 positions

LCD_BG_DOIT:
    call  LCD_1_WriteData              ; Display BG character
    call  LCD_Check_Ready              ; Wait for LCD to be ready

    dec   [X+BG_CHAR_LEN]              ; Dec Char count
    jnz   LCD_BG_LOOP1                 ; Do it all over again
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_DrawVBG
;
;  DESCRIPTION:
;     Draw a vertical bargraph on the LCD with the given parameters.
;
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;
;    A    => Starting row for bargraph 0 to 3
;   [X]   => Starting Column for bargraph 0 to 40+
;   [x-1] => Height of bargraph in chars 1 - 4
;   [X-2] => Position of pointer in segments 8 times height
;  RETURNS:
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;    If LCD_1_Init is not called before this function, the
;    bargraph will not be drawn properly.
;
;
;-----------------------------------------------------------------------------

; Stack offset constants
VBG_COL:            equ  0
VBG_CHAR_HEIGHT:    equ -1
VBG_SEG_HEIGHT:     equ -2

  LCD_1_DrawVBG:
 _LCD_1_DrawVBG:

    and   A,03h                        ; Make sure only rows 0 - 3 are valid
VBG_LOOP:
    push  A
    index LCD_ROW_OFFSET               ; Get row offset
    add   A,[X+VBG_COL]                ; Add column offset to position
    call  LCD_1_Control                ; Position Cursor
    cmp   [X+VBG_SEG_HEIGHT],00h       ; Check for zero segs
    jnz   VBG_NZ_SEG
    mov   A,' '                        ; Load space character
    jmp   VBG_WRITE_CHAR
VBG_NZ_SEG:
    cmp   [X+VBG_SEG_HEIGHT],09h       ; Check for full segment
    jnc   VBG_FULL_SEG
                                       ; Partial segment between 1 and 8
    mov   A,[X+VBG_SEG_HEIGHT]
    dec   A
    mov   [X+VBG_SEG_HEIGHT],00h       ; Zero segment height
    jmp   VBG_WRITE_CHAR

VBG_FULL_SEG:                          ; Bargaph
    sub   [X+VBG_SEG_HEIGHT],08h       ; Subtract full segment
    mov   A,07h                        ; Load full segment

VBG_WRITE_CHAR:                        ; Write character to display
    call  LCD_1_WriteData              ; Write value
    pop   A
    dec   A
    dec   [X+VBG_CHAR_HEIGHT]
    jnz   VBG_LOOP
    ret
.ENDSECTION

.SECTION
;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_InitVBG
;
;  DESCRIPTION:
;     Initialize the vertical bargraph characters.
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:  none
;
;  RETURNS:  none
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;    Only one type of bargraph (horizontal or vertical) may be used
;    at a time since they each require their own set of characters.
;
;  THEORY of OPERATION or PROCEDURE:
;    This function must be called prior to LCD_1_DrawVBG.
;
;-----------------------------------------------------------------------------

; Stack offset constants
VBGDATA_CTR:      equ    00h           ; Char data count stack offset
VBG_BYTES:        equ    01h           ; Byte counter stack offset

 LCD_1_InitVBG:
_LCD_1_InitVBG:
    mov   X,SP                         ; Get location of stack
    push  A                            ; Create 2 locations
    push  A

    mov   A,CG_RAM_OFFSET              ; Setup pointer
    call  LCD_1_Control                ; Position the CG pointer
    mov   [X+VBGDATA_CTR],01h          ; Reset data counter

 VBG_Loop1:                            ; loop once for each 8 characters
    mov   [X+VBG_BYTES],08h            ; Load cycle pointer
 VBG_Loop2:                            ; Loop once for each line in character (8 times)
    mov   A,[X+VBGDATA_CTR]
    cmp   A,[X+VBG_BYTES]
    jnc   VBG_SOLID
    mov   A,00h                        ; Empty line
    jmp   VBG_Load                     ; Jump to load the bargraph
VBG_SOLID:
    mov   A,FFh                        ; Load solid line
VBG_Load:
    call  LCD_Check_Ready
    call  LCD_1_WriteData              ; character data
    dec   [X+VBG_BYTES]                ; Dec byte counter
    jnz   VBG_Loop2                    ; End Loop 2
    inc   [X+VBGDATA_CTR]
    cmp   [X+VBGDATA_CTR],09h
    jnz   VBG_Loop1                    ; End Loop1

     pop  A
     pop  A
     mov  A,DISP_ON                    ; Turn on display, don't really
     call LCD_1_Control                ; need this.
     ret
.ENDSECTION

;-----------------------------------------------------------------------------
;  FUNCTION NAME: LCD_1_InitBG
;
;  DESCRIPTION:
;     Initialize horizontal bargraph characters
;
;-----------------------------------------------------------------------------
;
;  ARGUMENTS:
;     A = type  0 = full                   |||||||||........
;               1 = single vertical line   ..........|......
;
;  RETURNS:
;
;  SIDE EFFECTS:
;    REGISTERS ARE VOLATILE: THE A AND X REGISTERS MAY BE MODIFIED!
;    Only one type of bargraph (horizontal or vertical) may be used
;    at a time since they each require their own set of characters.
;
;  THEORY of OPERATION or PROCEDURE:
;    This function writes to the LCD character RAM to generate 8 custom
;    characters used to generated one of two horizontal bargraphs.
;
;-----------------------------------------------------------------------------
.LITERAL
 BG_TYPE1:: ; ....., |...., ||..., |||.., ||||., |||||, |||||
     DB    00h, 10h, 18h, 1Ch, 1Eh, 1Fh, 1Fh
 BG_TYPE2:: ; ....., |...., .|..., ..|.., ...|., ....|, .....
    DB  00h, 10h, 08h, 04h, 02h, 01h, 00h
.ENDLITERAL

.SECTION
; Stack offset constants
BGDATA_PTR:   equ    00h               ; Stack offsets
BGCHARS:      equ    01h
BGTYPE:       equ    02h

 LCD_1_InitBG:
_LCD_1_InitBG:
    mov   X,SP                         ; Get location of stack
    add   SP,3
    mov   [X+BGTYPE],A                 ; Store the bargraph type

    mov   A,CG_RAM_OFFSET              ; Setup pointer
    call  LCD_1_Control                ; Position the CG pointer
    mov   [X+BGDATA_PTR],00h           ; Reset pointer to BG data

 BG_Loop1:
    mov   [X+BGCHARS],08h              ; Load cycle pointer
 BG_Loop2:
    call  LCD_Check_Ready
    mov   A,[X+BGDATA_PTR]
    cmp   [X+BGTYPE],00h               ; Check which bargraph
    jnz   BG_OTHER
    index BG_TYPE1
    jmp   BG_Load
 BG_OTHER:
    index BG_TYPE2
 BG_Load:
    call  LCD_1_WriteData
    dec   [X+BGCHARS]                  ; Character builder counter
    jnz   BG_Loop2
    inc   [X+BGDATA_PTR]               ; Advance to next character
    cmp   [X+BGDATA_PTR],07h
    jnz   BG_Loop1

    add   SP,-3
    mov   A,DISP_ON
    call  LCD_1_Control
    ret
.ENDSECTION

ENDIF

; End of File LCD_1.asm
